import os

from abc import abstractmethod


class CredentialsResolver():

    def __init__(self, context):
        self.context = context
        self.credential_resolvers = [EnvNexusCredentials(context), SbtNexusCredentials(context)]
        self.credentials = None

    def resolve_nexus_credentials(self):
        if not self.credentials:
            self.credentials = self._resolve_credentials()
        return self.credentials

    def _resolve_credentials(self):
        for creds in self.credential_resolvers:
            if creds.exist():
                self.context.log("Resolved nexus credentials from: %s" % creds.describe())
                (user, password) = creds.load_creds()
                self.credentials = {"user": user, "password": password}
                return self.credentials
        self.context.log("Could not resolve nexus credentials!!! Downloading artifacts will fail")
        return None


class NexusCredentials():

    def __init__(self, context):
        self.context = context
        pass

    @abstractmethod
    def load_creds(self):
        pass

    @abstractmethod
    def exist(self):
        pass

    @abstractmethod
    def describe(self):
        pass


class SbtNexusCredentials(NexusCredentials):

    def __init__(self, context):
        NexusCredentials.__init__(self, context)
        self.sbt_location = context.config_value("sbtCredentialsFile", os.environ["HOME"] + "/.sbt/.credentials")

    def load_creds(self):
        creds = {key.strip(): value.strip() for (key, value) in map(lambda x: x.split("="), open(self.sbt_location, 'r').readlines())}
        return creds["user"], creds["password"]

    def exist(self):
        found = os.path.exists(self.sbt_location)
        if found:
            user, password = self.load_creds()
            found = user and password
        return found

    def describe(self):
        return "sbt credentials file (%s)" % self.sbt_location


class EnvNexusCredentials(NexusCredentials):

    def __init__(self, context):
        NexusCredentials.__init__(self, context)
        self.nexus_pass_env_var = context.config_value("nexusPasswordEnvironmentVar",  "NEXUS_PASS")
        self.nexus_user_env_var = context.config_value("nexusUserEnvironmentVar",  "NEXUS_USER")

    def load_creds(self):
        return os.environ.get(self.nexus_user_env_var, None), os.environ.get(self.nexus_pass_env_var, None)

    def exist(self):
        (nexus_user, nexus_password) = self.load_creds()
        found = nexus_user and nexus_password
        return found

    def describe(self):
        return "environment variables (%s and %s)" % (self.nexus_user_env_var, self.nexus_pass_env_var)



import os
import sys

sys.path.append(os.path.abspath(os.path.dirname(__file__) + '/' + '../servicemanager'))

# dont do this in production code, this is bad practice it would seem, only for tests
from servicemanager.actions import actions
from servicemanager.server import smserverlogic
from servicemanager.smcontext import ServiceManagerException

import time
import shutil
import unittest
import smcontext
from servicemanager.service.smplayservice import SmPlayServiceStarter

from servicemanager.serviceresolver import ServiceResolver

from BaseHTTPServer import HTTPServer
from SocketServer import ThreadingMixIn
from servicemanager.smcredentials import EnvNexusCredentials, CredentialsResolver, SbtNexusCredentials

class ThreadedHTTPServer(ThreadingMixIn, HTTPServer):
    """Handle requests in a separate thread."""


def set_up_and_clean_workspace():
    workspace_dir = os.path.join(os.path.dirname(__file__), "workspace")
    if os.path.exists(workspace_dir):
        shutil.rmtree(workspace_dir)
    os.mkdir(workspace_dir)
    os.environ["WORKSPACE"] = workspace_dir
    os.chdir(workspace_dir)


class TestFileServer(unittest.TestCase):
    def setUp(self):
        set_up_and_clean_workspace()

    def test_file_server(self):
        name = "FAKE_NEXUS"
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)

        context.kill("FAKE_NEXUS")
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])
        time.sleep(2)

        response1 = actions.start_one(context, name, True, False, None, port=None)
        self.assertTrue(response1)
        self.assertIsNotNone(context.get_service(name).status())
        response2 = actions.start_one(context, name, True, False, None, port=None)
        self.assertFalse(response2)
        context.kill(name)
        self.assertEqual(context.get_service(name).status(), [])


class TestNexus(unittest.TestCase):
    def setUp(self):
        set_up_and_clean_workspace()

    def test_nexus(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")

        # start fake nexus
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        response1 = actions.start_one(context, "FAKE_NEXUS", True, False, None, port=None)
        self.assertIsNotNone(context.get_service("FAKE_NEXUS").status())
        time.sleep(5)

        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        servicetostart = "PLAY_NEXUS_END_TO_END_TEST"
        actions.start_one(context, servicetostart, True, False, None, port=None)
        self.assertIsNotNone(context.get_service(servicetostart).status())
        context.kill(servicetostart)

        context.kill("FAKE_NEXUS")

        self.assertEqual(context.get_service(servicetostart).status(), [])
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])


class TestActions(unittest.TestCase):
    def setUp(self):
        set_up_and_clean_workspace()

    def test_start_and_stop_one(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        actions.start_one(context, "TEST_ONE", True, False, None, port=None)
        self.assertIsNotNone(context.get_service("TEST_ONE").status())
        context.kill("TEST_ONE")
        self.assertEqual(context.get_service("TEST_ONE").status(), [])

    def test_dropwizard_from_source(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        sm_application = smcontext.SmApplication(config_dir_override)
        context = smcontext.SmContext(sm_application, None, False, False)
        service_resolver = ServiceResolver(sm_application)

        servicetostart = "DROPWIZARD_NEXUS_END_TO_END_TEST"
        actions.start_and_wait(service_resolver, context, [servicetostart], False, False, None, port=None, seconds_to_wait=90)
        self.assertIsNotNone(context.get_service(servicetostart).status())
        context.kill(servicetostart)
        self.assertEqual(context.get_service(servicetostart).status(), [])

    def test_dropwizard_from_jar(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        sm_application = smcontext.SmApplication(config_dir_override)
        context = smcontext.SmContext(sm_application, None, False, False)
        service_resolver = ServiceResolver(sm_application)

        # start fake nexus
        actions.start_one(context, "FAKE_NEXUS", True, False, None, port=None)
        self.assertIsNotNone(context.get_service("FAKE_NEXUS").status())
        time.sleep(5)

        servicetostart = "DROPWIZARD_NEXUS_END_TO_END_TEST"
        actions.start_and_wait(service_resolver, context, [servicetostart], True, False, None, port=None, seconds_to_wait=90)
        self.assertIsNotNone(context.get_service(servicetostart).status())
        context.kill(servicetostart)
        context.kill("FAKE_NEXUS")
        time.sleep(5)
        self.assertEqual(context.get_service(servicetostart).status(), [])
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])

    def test_play_from_source(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        sm_application = smcontext.SmApplication(config_dir_override)
        context = smcontext.SmContext(sm_application, None, False, False)
        service_resolver = ServiceResolver(sm_application)

        servicetostart = "PLAY_NEXUS_END_TO_END_TEST"
        actions.start_and_wait(service_resolver, context, [servicetostart], False, False, None, port=None, seconds_to_wait=90)
        self.assertIsNotNone(context.get_service(servicetostart).status())
        context.kill(servicetostart)
        self.assertEqual(context.get_service(servicetostart).status(), [])

    def test_successful_play_from_jar_without_waiting(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        sm_application = smcontext.SmApplication(config_dir_override)
        context = smcontext.SmContext(sm_application, None, False, False)
        service_resolver = ServiceResolver(sm_application)

        context.kill_everything()
        time.sleep(5)

        response1 = actions.start_one(context, "FAKE_NEXUS", True, False, None, port=None)
        self.assertTrue(response1)
        self.assertIsNotNone(context.get_service("FAKE_NEXUS").status())
        time.sleep(5)

        try:
            servicetostart = ["PLAY_NEXUS_END_TO_END_TEST"]
            actions.start_and_wait(service_resolver, context, servicetostart, fatjar=True, release=False, proxy=None, port=None, seconds_to_wait=None)
        finally:
            context.kill_everything()

    def test_failing_play_from_jar(self):

        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        sm_application = smcontext.SmApplication(config_dir_override)
        context = smcontext.SmContext(sm_application, None, False, False)
        service_resolver = ServiceResolver(sm_application)

        context.kill_everything()
        time.sleep(5)

        response1 = actions.start_one(context, "FAKE_NEXUS", True, False, None, port=None)
        self.assertTrue(response1)
        self.assertIsNotNone(context.get_service("FAKE_NEXUS").status())
        time.sleep(5)

        try:
            servicetostart = ["BROKEN_PLAY_PROJECT"]
            actions.start_and_wait(service_resolver, context, servicetostart, fatjar=True, release=False, proxy=None, port=None, seconds_to_wait=2)
            self.fail("Did not expect the project to startup.")
        except ServiceManagerException as sme:
            self.assertEqual("Timed out starting service(s): BROKEN_PLAY_PROJECT", sme.message)
        finally:
            context.kill_everything()


    def test_start_and_stop_one_duplicate(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        response1 = actions.start_one(context, "TEST_ONE", True, False, None, port=None)
        self.assertTrue(response1)
        self.assertIsNotNone(context.get_service("TEST_ONE").status())
        response2 = actions.start_one(context, "TEST_ONE", True, False, None, port=None)
        self.assertFalse(response2)
        context.kill("TEST_ONE")
        self.assertEqual(context.get_service("TEST_ONE").status(), [])

    def test_assets_server(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        context.kill_everything()

        context.kill("FAKE_NEXUS")
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])
        time.sleep(2)

        # start fake nexus
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])
        response1 = actions.start_one(context, "FAKE_NEXUS", True, False, None, port=None)
        self.assertTrue(response1)
        self.assertIsNotNone(context.get_service("FAKE_NEXUS").status())
        time.sleep(5)

        actions.start_one(context, "PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND", True, False, None, port=None)
        self.assertIsNotNone(context.get_service("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND").status())
        context.kill("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND")
        context.kill("FAKE_NEXUS")
        time.sleep(15)

        self.assertEqual(context.get_service("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND").status(), [])
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])

    def test_wait_on_assets_server(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        sm_application = smcontext.SmApplication(config_dir_override)
        context = smcontext.SmContext(sm_application, None, False, False)
        service_resolver = ServiceResolver(sm_application)
        context.kill_everything()

        context.kill("FAKE_NEXUS")
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])
        time.sleep(2)

        # start fake nexus
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])
        response1 = actions.start_one(context, "FAKE_NEXUS", True, False, None, port=None)
        self.assertTrue(response1)
        self.assertIsNotNone(context.get_service("FAKE_NEXUS").status())
        time.sleep(5)

        actions.start_and_wait(service_resolver, context, ["PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND"], True, False, None, port=None, seconds_to_wait=5)
        self.assertIsNotNone(context.get_service("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND").status())
        context.kill("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND")
        context.kill("FAKE_NEXUS")
        time.sleep(15)

        self.assertEqual(context.get_service("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND").status(), [])
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])

    def test_python_server_offline(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, True, False)
        actions.start_one(context, "PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND", True, False, None, port=None)
        self.assertIsNotNone(context.get_service("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND").status())
        context.kill("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND")
        time.sleep(5)
        self.assertEqual(context.get_service("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND").status(), [])

class TestStartCommands(unittest.TestCase):

    def setUp(self):
        set_up_and_clean_workspace()

    def test_play_binary_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("PLAY_NEXUS_END_TO_END_TEST", True, False, None, port=None)
        #starter = SmPlayServiceStarter(context, "PLAY_NEXUS_END_TO_END_TEST", True, False, None, None, None, None)
        expected = [ './basicplayapp/bin/basicplayapp',
                                '-DProd.microservice.whitelist.useWhitelist=false',
                                '-DProd.mongodb.uri=mongodb://localhost:27017/auth',
                                '-J-Xmx256m',
                                '-J-Xms256m',
                                '-J-XX:MaxPermSize=128m',
                                '-Dhttp.port=8500',
                                '-Dservice.manager.serviceName=PLAY_NEXUS_END_TO_END_TEST',
                                '-Dservice.manager.runFrom=True']
        self.assertEqual(starter.get_start_command("BINARY"), expected)

    def test_play_source_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("PLAY_NEXUS_END_TO_END_TEST", True, False, None, port=None)
        expected = [ 'play', 'start -Dhttp.port=8500 -Dservice.manager.serviceName=PLAY_NEXUS_END_TO_END_TEST -Dservice.manager.runFrom=True -DFoo=false']
        self.assertEqual(starter.get_start_command("SOURCE"), expected)

    def test_dropwizard_binary_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("DROPWIZARD_NEXUS_END_TO_END_TEST", "foo", proxy=None)
        expected = [
            'java',
            '-Dfile.encoding=UTF8',
            '-Xmx64M',
            '-XX:+CMSClassUnloadingEnabled',
            '-XX:MaxPermSize=64m',
            '-Ddw.http.port=8080',
            '-Dservice.manager.serviceName=DROPWIZARD_NEXUS_END_TO_END_TEST',
            '-Dservice.manager.serviceName=DROPWIZARD_NEXUS_END_TO_END_TEST',
            '-Dservice.manager.runFrom=foo',
            '-jar',
            'dwtest-foo-shaded.jar',
            'server',
            'dev_config.yml']
        cmd = starter.get_start_command("BINARY")
        cmd[-1] = cmd[-1].split("/")[-1]
        cmd[0] = cmd[0].split("/")[-1]
        cmd[len(cmd) -3] = cmd[len(cmd) -3].split("/")[-1]
        self.assertEqual(cmd, expected)

    def test_dropwizard_source_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("DROPWIZARD_NEXUS_END_TO_END_TEST", "foo", proxy=None)
        expected = ['./startappfromcode.sh']
        self.assertEqual(starter.get_start_command("SOURCE"), expected)

    def test_python_binary_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND", "foo", proxy=None)
        expected = [ 'get_start_command() not implemented for this type of service - fork and make a pull request :)' ]
        cmd = starter.get_start_command("BINARY")
        self.assertEqual(cmd, expected)

    def test_python_source_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND", "foo", proxy=None)
        expected = ['get_start_command() not implemented for this type of service - fork and make a pull request :)']
        self.assertEqual(starter.get_start_command("SOURCE"), expected)

    def test_external_binary_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("FAKE_NEXUS", "foo", proxy=None)
        expected = [ 'get_start_command() not implemented for this type of service - fork and make a pull request :)' ]
        cmd = starter.get_start_command("BINARY")
        self.assertEqual(cmd, expected)

    def test_external_source_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        starter = context.get_service_starter("FAKE_NEXUS", "foo", proxy=None)
        expected = ['get_start_command() not implemented for this type of service - fork and make a pull request :)']
        self.assertEqual(starter.get_start_command("SOURCE"), expected)

class TestServerFunctionality(unittest.TestCase):
    def setUp(self):
        set_up_and_clean_workspace()

    def test_simple(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        context.kill_everything()
        server = smserverlogic.SmServer(smcontext.SmApplication(config_dir_override, None))
        request = dict()
        request["testId"] = "foo"
        request["services"] = [{"serviceName": "TEST_ONE", "runFrom": "SNAPSHOT"},
                               {"serviceName": "DROPWIZARD_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"},
                               {"serviceName": "PLAY_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"}]
        smserverlogic.SmStartRequest(server, request, True, False).process_request()
        self.assertIsNotNone(context.get_service("TEST_ONE").status())
        # stop does not currently work for extern
        # smserverlogic.SmStopRequest(SERVER, request).process_request()
        context.kill_everything()
        self.assertEqual(context.get_service("TEST_ONE").status(), [])

    def test_offline(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        context.kill_everything()
        server = smserverlogic.SmServer(smcontext.SmApplication(config_dir_override, None))
        request = dict()
        request["testId"] = "foo"
        request["services"] = [{"serviceName": "TEST_ONE", "runFrom": "SNAPSHOT"},
                               {"serviceName": "DROPWIZARD_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"},
                               {"serviceName": "PLAY_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"}]
        smserverlogic.SmStartRequest(server, request, True, False).process_request()
        self.assertIsNotNone(context.get_service("TEST_ONE").status())
        # stop does not currently work for extern
        # smserverlogic.SmStopRequest(SERVER, request).process_request()
        context.kill_everything()
        self.assertEqual(context.get_service("TEST_ONE").status(), [])
        request["testId"] = "foo2"
        smserverlogic.SmStartRequest(server, request, True, True).process_request()
        self.assertIsNotNone(context.get_service("TEST_ONE").status())
        # stop does not currently work for extern
        #smserverlogic.SmStopRequest(SERVER, request).process_request()
        context.kill_everything()
        self.assertEqual(context.get_service("TEST_ONE").status(), [])

    def test_ensure_multiple_instances_of_a_service_can_be_started_from_server(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        context.kill_everything()
        server = smserverlogic.SmServer(smcontext.SmApplication(config_dir_override, None))

        # start fake nexus
        self.assertEqual(context.get_service("FAKE_NEXUS").status(), [])
        response1 = actions.start_one(context, "FAKE_NEXUS", True, False, None, port=None)
        self.assertTrue(response1)
        self.assertIsNotNone(context.get_service("FAKE_NEXUS").status())
        time.sleep(5)

        first_request = dict()
        first_request["testId"] = "multiple-instance-unit-test-1"
        first_request["services"] = [{"serviceName": "TEST_ONE", "runFrom": "SNAPSHOT"},
                                     {"serviceName": "DROPWIZARD_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"},
                                     {"serviceName": "PLAY_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"}]
        request = smserverlogic.SmStartRequest(server, first_request, True, False)
        request.process_request()

        time.sleep(5)
        self.assertEqual(len(context.get_service("TEST_ONE").status()), 1)
        self.assertEqual(len(context.get_service("DROPWIZARD_NEXUS_END_TO_END_TEST").status()), 1)
        self.assertEqual(len(context.get_service("PLAY_NEXUS_END_TO_END_TEST").status()), 1)

        second_request = dict()
        second_request["testId"] = "multiple-instance-unit-test-2"
        second_request["services"] = [{"serviceName": "TEST_ONE", "runFrom": "SNAPSHOT"},
                                      {"serviceName": "DROPWIZARD_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"},
                                      {"serviceName": "PLAY_NEXUS_END_TO_END_TEST", "runFrom": "SNAPSHOT"}]
        smserverlogic.SmStartRequest(server, second_request, True, False).process_request()
        time.sleep(5)
        self.assertEqual(len(context.get_service("TEST_ONE").status()), 2)
        self.assertEqual(len(context.get_service("DROPWIZARD_NEXUS_END_TO_END_TEST").status()), 2)
        self.assertEqual(len(context.get_service("PLAY_NEXUS_END_TO_END_TEST").status()), 2)

        # stop does not currently work for extern
        # smserverlogic.SmStopRequest(SERVER, request).process_request()
        context.kill_everything()
        self.assertEqual(context.get_service("TEST_ONE").status(), [])
        context.kill("FAKE_NEXUS")


class TestConfiguration(unittest.TestCase):
    def setUp(self):
        set_up_and_clean_workspace()

    def test_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        application = smcontext.SmApplication(config_dir_override, None)
        self.assertEqual(len(application.services), 9)
        self.assertEqual(application.services["TEST_TEMPLATE"]["type"], "external")
        self.assertEqual(application.services["TEST_TEMPLATE"]["pattern"], "some.namespace=TEST_TEMPLATE")
        self.assertEqual(application.services["TEST_TEMPLATE"]["includeInStartAndStopAll"], False)
        self.assertEqual(application.services["TEST_TEMPLATE"]["some_parameter"], "value we expect")
        self.assertEqual(application.services["TEST_TEMPLATE"]["anestedvalue"]["anotherunique"], "value")
        self.assertEqual(application.services["TEST_TEMPLATE"]["anestedvalue"]["new"], "and this value")
        self.assertEqual(application.services["TEST_TEMPLATE"]["anestedvalue"]["override"], "will have this value")
        self.assertEqual(len(application.services["TEST_TEMPLATE"]), 8)


class TestServiceResolver(unittest.TestCase):
    def setUp(self):
        set_up_and_clean_workspace()

    def test_config(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        application = smcontext.SmApplication(config_dir_override, None)
        service_resolver = ServiceResolver(application)
        nexus_wildcard = service_resolver.resolve_services("PLAY_NEXU*TEST")
        self.assertTrue("PLAY_NEXUS_END_TO_END_TEST" in nexus_wildcard)
        self.assertEqual(1, len(nexus_wildcard))

        all_services = service_resolver.resolve_services("*")
        self.assertTrue("TEST_ONE" in all_services)
        self.assertTrue("TEST_TWO" in all_services)
        self.assertTrue("TEST_THREE" in all_services)
        self.assertTrue("TEST_TEMPLATE" in all_services)
        self.assertTrue("DROPWIZARD_NEXUS_END_TO_END_TEST" in all_services)
        self.assertTrue("PLAY_NEXUS_END_TO_END_TEST" in all_services)
        self.assertTrue("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND" in all_services)
        self.assertEqual(9, len(all_services))

        test_profile = service_resolver.resolve_services("TEST")
        self.assertTrue("TEST_ONE" in test_profile)
        self.assertTrue("TEST_TWO" in test_profile)
        self.assertTrue("TEST_THREE" in test_profile)
        self.assertTrue("TEST_TEMPLATE" in test_profile)
        self.assertEqual(4, len(test_profile))

        tests_without_one = service_resolver.resolve_services_from_array(["TEST", "-TEST_ONE"])
        self.assertFalse("TEST_ONE" in tests_without_one)
        self.assertTrue("TEST_TWO" in tests_without_one)
        self.assertTrue("TEST_THREE" in tests_without_one)
        self.assertTrue("TEST_TEMPLATE" in tests_without_one)
        self.assertEqual(3, len(tests_without_one))

        tests_without_one = service_resolver.resolve_services_from_array(["TEST_WILD_CARD_NEGATED_PROFILE"])
        self.assertTrue("TEST_ONE" in tests_without_one)
        self.assertFalse("TEST_TWO" in tests_without_one)
        self.assertTrue("TEST_THREE" in tests_without_one)
        self.assertTrue("TEST_TEMPLATE" in tests_without_one)
        self.assertEqual(3, len(tests_without_one))

        nothing = service_resolver.resolve_services_from_array(["*", "-*"])
        self.assertEqual(0, len(nothing))

        test_template = service_resolver.resolve_services("TEST_WILD_CARD_PROFILE")
        self.assertTrue("TEST_TEMPLATE" in test_template)
        self.assertEqual(1, len(test_template))


class TestConfig(unittest.TestCase):
    def test_runfrom_override(self):
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        python_server = context.get_service("PYTHON_SIMPLE_SERVER_ASSETS_FRONTEND")
        run_from = context.get_run_from_service_override_value_or_use_default(python_server, "SHOULD_BE_OVERWRITTEN")
        self.assertEqual(run_from, "RELEASE")


class TestCredentialsResolver(unittest.TestCase):

    defaultEnv = os.environ.copy()

    def test_env_nexus_creds(self):
        os.environ.clear()
        os.environ.update(self.defaultEnv)
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        creds = EnvNexusCredentials(context)
        resolver = CredentialsResolver(context)

        os.environ.setdefault("NEXUS_PASS", "lame")
        os.environ.setdefault("NEXUS_USER", "lame_user")

        self.assertEqual(creds.load_creds(), ("lame_user", "lame"))
        self.assertEqual(resolver.resolve_nexus_credentials(), {"user": "lame_user", "password": "lame"})


    def test_env_nexus_creds_from_config(self):
        os.environ.clear()
        os.environ.update(self.defaultEnv)
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        context.application.config["nexusPasswordEnvironmentVar"] = "A_FREAKIN_PASS_ENV_VAR"
        context.application.config["nexusUserEnvironmentVar"] = "A_FREAKIN_USER_ENV_VAR"
        creds = EnvNexusCredentials(context)
        resolver = CredentialsResolver(context)

        os.environ.setdefault("A_FREAKIN_PASS_ENV_VAR", "lame_from_conf")
        os.environ.setdefault("A_FREAKIN_USER_ENV_VAR", "lame_user_from_conf")

        self.assertEqual(creds.load_creds(), ("lame_user_from_conf", "lame_from_conf"))
        self.assertEqual(resolver.resolve_nexus_credentials(), {"user": "lame_user_from_conf", "password": "lame_from_conf"})

    def test_sbt_nexus_creds_from_config(self):
        os.environ.clear()
        os.environ.update(self.defaultEnv)
        config_dir_override = os.path.join(os.path.dirname(__file__), "conf")
        context = smcontext.SmContext(smcontext.SmApplication(config_dir_override), None, False, False)
        context.application.config["sbtCredentialsFile"] = os.path.dirname(__file__) + "/.sbt/.credentials"

        creds = SbtNexusCredentials(context)
        resolver = CredentialsResolver(context)

        self.assertEqual(creds.load_creds(), ("sbt", "sbt_creds"))
        self.assertEqual(resolver.resolve_nexus_credentials(), {"user": "sbt", "password": "sbt_creds"})


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import os
import shutil
import zipfile
import stat
import time
import json
import copy

from servicemanager.subprocess import Popen
from ..service.smservice import SmMicroServiceStarter
from smjvmservice import SmJvmService, SmJvmServiceStarter
from ..smfile import force_chdir, force_pushdir, remove_if_exists, remove_folder_if_exists, makedirs_if_not_exists
from ..smnexus import SmNexus
from ..actions.colours import BColors

from servicemanager import subprocess


b = BColors()


class SmPlayServiceStarter(SmJvmServiceStarter):

    PLAY_PROCESS_STARTUP_TIMEOUT_SECONDS = 120

    def __init__(self, context, service_name, run_from, port, classifier, service_mapping_ports, version, proxy):
        SmMicroServiceStarter.__init__(self, context, service_name, "play", run_from, port, classifier, service_mapping_ports, version, proxy)

        if not self.port:
            self.port = self.service_data["defaultPort"]

    def _build_extra_params(self):
        extra_params = ["-Dhttp.port=%d" % self.port]
        extra_params += self.process_arguments()
        # Features are so specific - should this be in config?
        if self.context.features:
            for feature in self.context.features:
                extra_params += ["-Dfeature.%s=true" % feature]

        service_config_key = "Dev.microservice.services"

        if self.context.is_test:
            if self.service_data.get("hasMongo", False):
                extra_params += ["-DDev.microservice.mongodb.uri=mongodb://localhost:27017/%s-%s" % (self.context.database_name_prefix, self.service_name)]

        if self.service_mapping_ports and self.service_data.get("hasServiceMappings", False):
            for dependent_service_name in self.service_mapping_ports:
                service_config_key_with_prefix = service_config_key
                if self.service_name == "FEGOVUK":
                    service_config_key_with_prefix = "govuk-tax.Dev.services"
                extra_params += [
                    "-D%s.%s.host=localhost" % (service_config_key_with_prefix, dependent_service_name),
                    "-D%s.%s.port=%d" % (service_config_key_with_prefix, dependent_service_name, self.service_mapping_ports[dependent_service_name])
                ]

        if self.proxy:
            proxy_config = self.proxy.split(":")
            print "Starting service with proxy, '" + str(self.proxy) + "'"
            extra_params += [
                "-Dhttp.proxyHost=" + proxy_config[0],
                "-Dhttp.proxyPort=" + proxy_config[1]
            ]

        return extra_params

    def get_start_command(self, run_from):
        if run_from == "SOURCE":
            source_cmd = copy.copy(self.service_data["sources"]["cmd"])
            source_cmd[-1] = source_cmd[-1] + " " +  " ".join(self.sbt_extra_params())
            return source_cmd
        else:
            return self.service_data["binary"]["cmd"] + self._build_extra_params()

    def start_from_binary(self):
        microservice_target_path = self.context.get_microservice_target_path(self.service_name)
        force_chdir(microservice_target_path)

        if not self.context.offline:
            nexus = SmNexus(self.context, self.service_name)
            nexus.download_jar_if_necessary(self.run_from, self.version)

        unzip_dir = self._unzip_play_application()
        parent, _ = os.path.split(unzip_dir)
        force_pushdir(parent)

        cmd_with_params = self.get_start_command("BINARY")
        if os.path.exists(cmd_with_params[0]):
            os.chmod(cmd_with_params[0], stat.S_IRWXU)
        else:
            print b.fail + "ERROR: unable to chmod on non existent file '" + parent + cmd_with_params[0] + "'" + b.endc

        makedirs_if_not_exists("logs")

        print(cmd_with_params)

        with open("logs/stdout.txt", "wb") as out, open("logs/stderr.txt", "wb") as err:
            popen_output = Popen(cmd_with_params, env=os.environ.copy(), stdout=out, stderr=err, close_fds=True)
            if popen_output.returncode == 1:
                print b.fail + "ERROR: could not start '" + self.service_name + "' " + b.endc
            return popen_output.pid

    def _unzip_play_application(self):
        service_data = self.service_data
        microservice_zip_path = self.context.application.workspace + service_data["location"] + "/target/"
        force_pushdir(microservice_zip_path)
        zip_filename = service_data["binary"]["artifact"] + ".zip"

        unzipped_dir = SmPlayService.unzipped_dir_path(self.context, service_data["location"])
        remove_folder_if_exists(unzipped_dir)

        os.makedirs(unzipped_dir)
        zipfile.ZipFile(zip_filename, 'r').extractall(unzipped_dir)

        folder = os.listdir(unzipped_dir)[0]
        target_dir = unzipped_dir + "/" + service_data["binary"]["destinationSubdir"]
        shutil.move(unzipped_dir + "/" + folder, target_dir)

        return target_dir

    def sbt_extra_params(self):
        sbt_extra_params = self._build_extra_params()

        if "extra_params" in self.service_data["sources"]:
            sbt_extra_params += self.service_data["sources"]["extra_params"]

        return sbt_extra_params

    def start_from_sources(self):
        sbt_extra_params = self.sbt_extra_params()

        service_data = self.context.service_data(self.service_name)
        microservice_path = self.context.application.workspace + service_data["location"]
        curr_dir = force_pushdir(microservice_path)

        env_copy = os.environ.copy()
        env_copy["SBT_EXTRA_PARAMS"] = " ".join(sbt_extra_params) # TODO: not needed i think anymore...

        makedirs_if_not_exists("logs")

        with open("logs/stdout.txt", "wb") as out, open("logs/stderr.txt", "wb") as err:
            process = Popen(self.get_start_command("SOURCE"), env=env_copy, stdout=out, stderr=err, stdin=subprocess.PIPE)
            process.stdin.close()
            if process.returncode == 1:
                print b.fail + "ERROR: could not start '" + self.service_name + "' " + b.endc
            return process.pid # Note: This is the parent pid


class SmPlayService(SmJvmService):

    @staticmethod
    def unzipped_dir_path(context, location):
        return context.application.play_extraction_dir + location + "_" + context.instance_id

    def __init__(self, context, service_name):
        SmJvmService.__init__(self, context, service_name, "play")
        self.default_port = self.required_data("defaultPort")
        self.healthcheck = self.required_data("healthcheck")

    def post_stop(self):
        pass

    def clean_up(self):
        unzip_path = SmPlayService.unzipped_dir_path(self.context, self.service_data["location"])
        remove_folder_if_exists(unzip_path)

    def get_details_url(self):
        return "http://localhost:${port}/admin/details"

    def get_port_argument(self):
        return "http.port"

    def get_running_healthcheck_port(self, process):
        return process.extract_integer_argument('-D%s=(\d*)' % self.get_port_argument(), self.default_port)



#!/usr/bin/env python
import os
import shutil
import zipfile
import signal
import time
import re

import requests

from servicemanager import subprocess
from smservice import SmService, SmMicroServiceStarter, SmServiceStatus
from servicemanager.smprocess import SmProcess
from servicemanager.smfile import force_chdir, force_pushdir, remove_if_exists, remove_folder_if_exists, makedirs_if_not_exists
from servicemanager.smnexus import SmNexus
from servicemanager.smrepo import clone_repo_if_requred


class SmPythonServiceStarter(SmMicroServiceStarter):

    PROCESS_STARTUP_TIMEOUT_SECONDS = 90

    def __init__(self, context, service_name, run_from, port, classifier, service_mapping_ports, version, proxy):
        SmMicroServiceStarter.__init__(self, context, service_name, "assets", run_from, port, classifier, service_mapping_ports, version, None)

        if not self.port:
            self.port = self.service_data["defaultPort"]

    def process_arguments(self):
        pass

    def _start_from_binary(self):
        assets_target_path = self.context.get_microservice_target_path(self.service_name)
        assets_path = self.context.application.workspace + self.service_data["location"]

        force_chdir(assets_path)
        remove_if_exists("RUNNING_FROM")

        force_chdir(assets_target_path)

        if not self.context.offline:
            nexus = SmNexus(self.context, self.service_name)
            versions = nexus.get_all_versions(self.version, self.run_from)
            for version in versions:
                nexus.download_jar_if_necessary(self.run_from, version)
            self._unzip_assets(versions)

        cmd_with_params = self.service_data["binary"]["cmd"]
        makedirs_if_not_exists("logs")
        with open("logs/stdout.txt", "wb") as out, open("logs/stderr.txt", "wb") as err:
            return subprocess.Popen(cmd_with_params[0].split(), shell=False, env=os.environ.copy(), stdout=out, stderr=err, close_fds=True).pid

    def _start_from_sources(self):

        service_data = self.context.service_data(self.service_name)
        assets_path = self.context.application.workspace + service_data["location"]

        cmd_with_params = self.service_data["sources"]["cmd"]
        force_pushdir(assets_path)
        run_from_file = open("RUNNING_FROM", 'w')
        run_from_file.write(self.run_from)
        run_from_file.close()

        makedirs_if_not_exists("logs")
        seconds_remaining = SmPythonServiceStarter.PROCESS_STARTUP_TIMEOUT_SECONDS

        with open("logs/stdout.txt", "wb") as out, open("logs/stderr.txt", "wb") as err:
            subprocess.Popen(cmd_with_params, shell=False, env=os.environ.copy(), stdout=out, stderr=err, close_fds=True)

        while seconds_remaining > 0 and not len(SmProcess.processes_matching("grunt")) > 0:
            time.sleep(1)
            seconds_remaining -= 1
            if seconds_remaining < 10 or seconds_remaining % 5 == 0:
                self.log("Waiting for Assets service to start: %s second%s before timeout" % (
                    seconds_remaining, "s" if seconds_remaining > 1 else ""))
        if len(SmProcess.processes_matching("grunt")) == 1:
            process = SmProcess.processes_matching("grunt")
            for i, v in enumerate(process):
                    return v.pid

    def _unzip_assets(self, versions):
        service_data = self.service_data
        assets_zip_path = self.context.get_microservice_target_path(self.service_name)
        force_pushdir(assets_zip_path)
        unzipped_dir = "assets"
        if not os.path.exists(unzipped_dir):
            os.makedirs(unzipped_dir)

        for version in versions:
            zip_filename = service_data["binary"]["artifact"] + "-" + version + ".zip"
            extracted_dir = version
            if not os.path.exists(assets_zip_path + "/assets/" + extracted_dir):
                os.makedirs(extracted_dir)
                zipfile.ZipFile(zip_filename, 'r').extractall(extracted_dir)
                shutil.move(extracted_dir, unzipped_dir)
        target_dir = unzipped_dir
        return target_dir

    def start(self):
        if self.run_from == "SOURCE" and "sources" in self.service_data:
            if not clone_repo_if_requred(self):
                # TODO - should this just return? or throw an exception, or return None explicitly?
                return

            return self._start_from_sources()
        else:
            return self._start_from_binary()


class SmPythonService(SmService):

    @staticmethod
    def unzipped_dir_path(context, location):
        return context.application.play_extraction_dir + location + "_" + context.instance_id

    def __init__(self, context, service_name):

        SmService.__init__(self, context, service_name, "assets")
        self.default_port = self.required_data("defaultPort")
        self.healthcheck = self.required_data("healthcheck")
        self.pattern = SmPythonService.get_pattern(self)

    def stop(self):

        ps_command = "ps axo pid,command | grep '%s' | grep -v 'grep' | awk '{print $1}'" % SmPythonService.get_pattern(self)

        ps = subprocess.Popen(ps_command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        pid_values = map(int, ps.stdout.read().split("\n")[:-1])

        if len(pid_values) == 0:
            return

        self.log("Killing service '%s' (pid = %s)..." % (self.service_data["name"], str(pid_values)))

        for pid_int in pid_values:
            os.kill(pid_int, signal.SIGINT)
            self.log("PID  %d killed" % pid_int)


    def clean_up(self):
        unzip_path = SmPythonService.unzipped_dir_path(self.context, self.service_data["location"])
        remove_folder_if_exists(unzip_path)

    def request_running_service_details_on_default_port(self):
        # TODO: implement git revision stuff for python or remove the feature
        return None

    def status(self):
        processes = SmProcess.processes_matching(SmPythonService.get_pattern(self))

        if len(processes) == 0:
            return []

        if len(processes) > 1:
            self.log("POSSIBLE PROBLEM: Found more than one process")

        def _status_for_process(process):
            healthcheck = SmServiceStatus.HEALTHCHECK_PASS if self.run_healthcheck(None) else SmServiceStatus.HEALTHCHECK_BOOT
            return SmServiceStatus.for_process(self.service_name, process, self.default_port, "", "", "", healthcheck)

        return map(_status_for_process, processes)

    def run_healthcheck(self, process):
        port = self.default_port

        healthcheck_url = self.service_data["healthcheck"]["url"].replace("${port}", str(port))

        healthcheck_response_regex = self.service_data["healthcheck"]["response"]
        try:
            ping_response = requests.get(healthcheck_url)
            response_text = str(ping_response.status_code)
            return re.search(healthcheck_response_regex, response_text) is not None
        except requests.RequestException:
            return False

    def is_started_on_default_port(self):
        return len(SmProcess.processes_matching(SmPythonService.get_pattern(self))) > 0

    def get_pattern(self):
        return self.service_data["pattern"]


import os
import sys
import time
import urllib
import hashlib
import urllib2
import base64
from xml.dom.minidom import parse

import requests

from servicemanager.smfile import remove_if_exists
from actions.colours import BColors


b = BColors()


class SmNexus():

    def __init__(self, context, service_name):
        self.context = context
        self.service_name = service_name
        self.service_type = context.service_type(service_name)

    @staticmethod
    def _report_hook(count, block_size, total_size):
        global start_time
        if count == 0:
            start_time = time.time()
            return
        duration = time.time() - start_time
        progress_size = int(count * block_size)
        speed = int(progress_size / (1024 * duration))
        percent = int(count * block_size * 100 / total_size)
        sys.stdout.write("\r%d%%, %d MB, %d KB/s, %d seconds passed" %
                         (percent, progress_size / (1024 * 1024), speed, duration))
        sys.stdout.flush()

    def _create_nexus_extension(self):
        if self.service_type == "play" or self.service_type == "assets":
            return ".zip"
        else:
            return "-shaded.jar"

    @staticmethod
    def _md5_if_exists(path):
        if os.path.exists(path):
            return hashlib.md5(open(path, 'rb').read()).hexdigest()
        else:
            return 0

    def resolve_credentials(self):
        return self.context.credentials.resolve_nexus_credentials()

    def _header_credentials(self):
        credentials = self.resolve_credentials()
        return credentials["user"] + ":" + credentials["password"]

    def _url_credentials(self):
        credentials = self.resolve_credentials()
        return urllib.quote_plus(credentials['user']) + ":" + urllib.quote_plus(credentials["password"])

    def _download_from_nexus(self, nexus_path, shaded_jar, show_progress):
        url = self._get_protocol() + "://" + self._url_credentials() + "@" + nexus_path
        if show_progress:
            urllib.urlretrieve(url, shaded_jar, SmNexus._report_hook)
            print("\n")
        else:
            urllib.urlretrieve(url, shaded_jar)

    def _is_valid_repository(self, repository, data):
        repository_id = data.getElementsByTagName("latest" + repository + "RepositoryId")[0].firstChild.nodeValue
        repo_mappings = self.context.config_value("nexus")["repoMappings"]
        if not repository_id in repo_mappings.values():
            self.context.log("The repositoryId " + repository_id + " is not in: " + str(repo_mappings.values()))
            sys.exit(-1)

    def _find_version_in_dom(self, repository, dom):
        latest = "latest" + repository
        try:
            data = dom.getElementsByTagName("artifact")[0]
        except:
            self.context.log("Unable to get latest version from nexus")
            return None

        self._is_valid_repository(repository, data)
        return data.getElementsByTagName(latest)[0].firstChild.nodeValue

    def _get_protocol(self):
        protocol = "https"
        if "protocol" in self.context.config_value("nexus"):
            protocol = self.context.config_value("nexus")["protocol"]
        return protocol

    def _get_version_info_from_nexus(self, artifact):
        lucene_nexus = self._get_protocol() + "://" + self.context.config_value("nexus")["host"] + "/service/local/lucene/search?a=" + artifact
        request = urllib2.Request(lucene_nexus)
        base64string = base64.encodestring(self._header_credentials()).replace('\n', '')
        request.add_header("Authorization", "Basic %s" % base64string)
        response = urllib2.urlopen(request)
        dom = parse(response)
        response.close()
        return dom

    def _find_latest_version(self, repository, artifact):
        version_env_var = None
        if "versionEnv" in self.context.service_data(self.service_name):
            version_env_var = self.context.service_data(self.service_name)["versionEnv"]

        try:
            version = os.environ[version_env_var]
        except Exception:
            dom = self._get_version_info_from_nexus(artifact)
            version = self._find_version_in_dom(repository, dom)
        return version

    def get_all_versions(self, version, run_from):
        binary = self.context.service_data(self.service_name)["binary"]
        repo_mappings = self.context.config_value("nexus")["repoMappings"]
        if run_from == "RELEASE":
            url_type_repository = repo_mappings["RELEASE"]
        else:
            url_type_repository = repo_mappings["SNAPSHOT"]
        versions = []
        nexus_url = self.context.application.nexus_repo_host + binary["nexus-api"] + url_type_repository + "/content/" + binary["groupId"] + binary["artifact"] + "/"
        print nexus_url
        url = self._get_protocol() + "://" + self._url_credentials() + "@" + nexus_url
        headers = {'Accept': 'application/json'}
        response = requests.get(url, headers=headers)
        json_object = response.json()
        for release in json_object["data"]:
            #don't want to download any maven metadata
            caste = str(release["text"])
            if not "maven-metadata" in caste:
                print release["text"]
                versions.append(release["text"])
        return versions

    def download_jar_if_necessary(self, run_from, version):
        binary = self.context.service_data(self.service_name)["binary"]
        nexus_host = self.context.application.nexus_repo_host
        artifact = binary["artifact"]
        filename = self.context.get_jar_filename(self.service_name, run_from)
        microservice_target_path = self.context.get_microservice_target_path(self.service_name)
        repo_mappings = self.context.config_value("nexus")["repoMappings"]
        if run_from == "RELEASE":
            type_repository = "Release"
            url_type_repository = repo_mappings["RELEASE"]
        else:
            type_repository = "Snapshot"
            url_type_repository = repo_mappings["SNAPSHOT"]

        if not version:
            version = self._find_latest_version(type_repository, artifact)

        if version:
            nexus_extension = self._create_nexus_extension()
            nexus_filename = artifact + "-" + version + nexus_extension
            md5_filename = nexus_filename + ".md5"
            group_id = binary["groupId"]
            nexus_url = nexus_host + binary["nexus"] + url_type_repository + "/" + group_id + artifact + "/" + version + "/"
            #first download the md5 file in order to determine if new artifact download is required
            self._download_from_nexus(nexus_url + md5_filename, microservice_target_path + md5_filename, False)
            if self.service_type == "assets":
                if self._md5_if_exists(microservice_target_path + nexus_filename) != open(microservice_target_path + md5_filename, 'r').read():
                    remove_if_exists(microservice_target_path + filename)
                    self.context.log("Downloading binary for '" + self.service_name + "': " + nexus_filename)
                    self._download_from_nexus(nexus_url + nexus_filename, nexus_filename, self.context.show_progress)
            else:
                if self._md5_if_exists(microservice_target_path + filename) != open(microservice_target_path + md5_filename, 'r').read():
                    remove_if_exists(microservice_target_path + filename)
                    self.context.log("Downloading binary for '" + self.service_name + "': " + nexus_filename)
                    self._download_from_nexus(nexus_url + nexus_filename, filename, self.context.show_progress)
            os.remove(microservice_target_path + md5_filename)
        else:
            print b.warning + "WARNING: Due to lack of version data from nexus you may not have an up to date version..." + b.endc



import os
import shutil


def remove_if_exists(path):
    if os.path.exists(path):
        os.remove(path)


def remove_folder_if_exists(path):
    if os.path.exists(path):
        shutil.rmtree(path)


def makedirs_if_not_exists(path):
    if not os.path.exists(path):
        os.makedirs(path)


def force_chdir(path):
    makedirs_if_not_exists(path)
    os.chdir(path)


def force_pushdir(path):
    curdir = os.getcwd()
    makedirs_if_not_exists(path)
    os.chdir(path)
    return curdir



#!/usr/bin/env python
import os
import time
import calendar
import glob

from servicemanager import subprocess
from servicemanager.smcontext import ServiceManagerException
from servicemanager.smprocess import SmProcess
from servicemanager.service.smplayservice import SmPlayService

def start_one(context, service_name, fatjar, release, proxy, port=None):
    if release:
        run_from = "RELEASE"
    elif fatjar:
        run_from = "SNAPSHOT"
    else:
        run_from = "SOURCE"

    existing_service_status = context.get_service(service_name).status()

    if len(existing_service_status) > 0:
        print "There is already: '" + str(len(existing_service_status)) + "' instance(s) of the service: '" + service_name + "' running"
        return False

    if context.start_service(service_name, run_from, proxy, None, None, port):
        if context.get_service(service_name).is_started_on_default_port():
            print "Started: " + service_name
            return True

    return False

def get_start_cmd(context, service_name, fatjar, release, proxy, port=None):
    if release:
        run_from = "RELEASE"
    elif fatjar:
        run_from = "SNAPSHOT"
    else:
        run_from = "SOURCE"

    starter = context.get_service_starter(service_name, run_from, proxy, None, None, port)
    return starter.get_start_command(run_from)

def stop_profile(context, profile):
    for service_name in context.application.services_for_profile(profile):
        context.kill(service_name)


def _now():
    return int(calendar.timegm(time.gmtime()))


def _wait_for_services(context, service_names, seconds_to_wait):

    waiting_for_services = []

    for service_name in service_names:
        if "healthcheck" in context.service_data(service_name):
            waiting_for_services += [context.get_service(service_name)]

    if not seconds_to_wait:
        seconds_to_wait = 0

    end_time = _now() + seconds_to_wait

    while waiting_for_services and _now() < end_time:

        services_to_check = list(waiting_for_services)

        for service in services_to_check:

            if _now() >= end_time:
                break

            processes = SmProcess.processes_matching(service.pattern)
            if all(map(service.run_healthcheck, processes)):
                print "Service '%s' has started successfully" % service.service_name
                waiting_for_services.remove(service)
            else:
                seconds_remaining = end_time - _now()
                if seconds_remaining % 5 == 0 or seconds_remaining < 10:
                    print "Waiting for %s to start, %s second%s before timeout" % (service.service_name, seconds_remaining, "s" if seconds_to_wait != 1 else "")

        if waiting_for_services:
            time.sleep(1)

    if waiting_for_services:
        services_timed_out = []
        for service in waiting_for_services:
            services_timed_out += [service.service_name]
        raise ServiceManagerException("Timed out starting service(s): %s" % ", ".join(services_timed_out))

def clean_logs(context, service_name):
    data = context.service_data(service_name)
    if "location" in data:
        sources_logs = context.application.workspace + data["location"] + "/logs/*.log*"
        fatjar_logs = context.application.workspace + data["location"] + "/target/logs/*.log*"

        num_files = _remove_files_wildcard(sources_logs)
        num_files += _remove_files_wildcard(fatjar_logs)
        print "Removed %d log files in %s" % (num_files, service_name)


def _remove_files_wildcard(files):
    r = glob.glob(files)
    for i in r:
        os.remove(i)
    return len(r)


def overridden_port(services, port):
    if len(services) == 1 and port is not None:
        return port
    else:
        return None


def _get_running_process_args(context, service_name):

    service = context.get_service(service_name)

    pattern = service.get_pattern()

    if service.is_started_on_default_port():
        command = "ps -eo args | egrep '%s' | egrep -v 'egrep %s' | awk '{{print  } }'" % (pattern, pattern)
        ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
        ps_output = ps_command.stdout.read()

        if ps_output:
            return ps_output.split("\n")[:-1][0]

    return ""


def _get_git_rev(context, service_name):
    details = context.get_service(service_name).request_running_service_details_on_default_port()

    if details:
        return details.get("Git-Head-Rev", "")

    return ""

def display_info(context, service_name):
    arguments = _get_running_process_args(context, service_name)
    git_revision = _get_git_rev(context, service_name)
    print
    print "| %s" % service_name
    print "| %s" % arguments
    print "| %s" % git_revision
    comments = ""
    if git_revision == "":
        comments += "(No Details) "
    if arguments == "":
        comments += "(Not Running) "
    print "| " + comments


def start_and_wait(service_resolver, context, start, fatjar, release, proxy, port, seconds_to_wait):

    all_services = service_resolver.resolve_services_from_array(start)
    for service_name in all_services:
        if context.has_service(service_name):
            start_one(context, service_name, fatjar, release, proxy, overridden_port(start, port))
        else:
            print "The requested service %s does not exist" % service_name

    if seconds_to_wait:
        _wait_for_services(context, all_services, seconds_to_wait)

    print "All services passed healthcheck"


def get_log_file(context, service_name):

    def mtime(path):
        try:
            return os.path.getmtime(path)
        except os.error:
            return 0

    data = context.service_data(service_name)
    if "location" in data:
        logs = [context.application.workspace + data["location"] + "/logs/stdout.txt",
                context.application.workspace + data["location"] + "/target/logs/stdout.txt",
                SmPlayService.unzipped_dir_path(context, data["location"]) + "/logs/stdout.txt"]
        if not any([os.path.exists(log) for log in logs]):
            raise ServiceManagerException("Cannot find log files for %s" % service_name)
        else:
            return sorted(logs, key=mtime, reverse=True)[0]
    else:
        raise ServiceManagerException("Cannot find a location for %s" % service_name)




#!/usr/bin/env python

from servicemanager.thirdparty.prettytable import PrettyTable
from servicemanager.service.smservice import SmServiceStatus
from servicemanager.smcontext import ServiceManagerException
from servicemanager.actions.colours import BColors

import os

def _format_healthcheck_status(healthcheck):
    b = BColors()
    if healthcheck == SmServiceStatus.HEALTHCHECK_PASS:
        return b.bold + b.okgreen + "PASS" + b.endc
    elif healthcheck == SmServiceStatus.HEALTHCHECK_BOOT:
        return b.bold + b.warning + "BOOT" + b.endc
    elif healthcheck == SmServiceStatus.HEALTHCHECK_NONE:
        return b.okblue + "NONE" + b.endc
    else:
        raise ServiceManagerException("Unknown healthcheck status: %s" % healthcheck)


def _service_status_to_row(status):
    return [
        status.service_name,
        status.ppid,
        status.pid,
        status.uptime,
        status.mem,
        status.port,
        status.test_id,
        status.run_from,
        status.features,
        _format_healthcheck_status(status.healthcheck)
    ]


def dostatus(context, services, show_down_services, clear_before_print=False):
    b = BColors()
    up_processes_table = PrettyTable()
    up_processes_table.field_names = ["name", "ppid", "pid", "uptime", "mem", "port", "test id", "run from", "features", "healthcheck"]
    up_processes_table.align = "r"
    up_processes_table.align["name"] = "l"
    up_processes_table.sortby = "name"
    up_processes_table.align["test id"] = "l"
    up_processes_table.align["run from"] = "l"
    up_processes_table.align["features"] = "l"

    down_processes_table = PrettyTable()
    down_processes_table.field_names = ["name", "healthcheck"]
    down_processes_table.align = "r"
    down_processes_table.align["name"] = "l"
    down_processes_table.sortby = "name"

    if len(services) == 0:
        services = context.application.services

    for service_name in services:
        responses = context.get_service(service_name).status()
        if responses:
            for response in responses:
                up_processes_table.add_row(_service_status_to_row(response))
        elif show_down_services:
            down_processes_table.add_row([service_name, b.bold + b.fail + "DOWN" + b.endc])

    # Perhaps there is a better way of doing this by clearing the buffer
    # but this will do the trick for now
    if clear_before_print:
        os.system("clear") # Linux only I think, but thats all we currently support, so... I guess that's ok

    print "Running:"
    print up_processes_table
    if show_down_services:
        print "Down:"
        print down_processes_table



#!/usr/bin/env python

"""
   Life's short, Python more.
   bug report to bazhen.csy@taobao.com.
"""

import sys
import threading


class AtomicInteger:

    def __init__(self, integer, minimum=0, maximum=sys.maxint):
        self.counter = integer
        self.min = minimum
        self.max = maximum
        self.lock = threading.RLock()
        return

    def increment_and_get(self):
        self.lock.acquire()

        if self.counter >= self.max:
            self.counter = self.min
        else:
            self.counter += 1

        value = self.counter
        self.lock.release()
        return value

    def decrement_and_get(self):
        self.lock.acquire()

        if self.counter <= self.min:
            self.counter = self.max
        else:
            self.counter -= 1

        value = self.counter
        self.lock.release()
        return value

    def set_value(self, integer):
        self.lock.acquire()
        self.counter = integer
        value = self.counter
        self.lock.release()
        return value

    def get(self):
        return self.counter

if __name__ == r"__main__":
    sys.exit(0)


# Format string may contain two special values '$s' will be replaced by 's' if the list has more than one value, $list will be replaced by the list.
def pretty_print_list(format_string, values, default_if_empty=""):

    if not values:
        return default_if_empty

    if len(values) == 1:
        s = ""
        list_string = "'%s'" % values[0]
    else:
        s = "s"
        list_string = ", ".join("'{0}'".format(w) for w in values[:-1]) + (" and '%s'" % values[-1])

    return format_string.replace("$s", s).replace("$list", list_string)


def if_not(value, default_value):
    return value if value else default_value


def unify_lists(*lists):
    result = []

    for a_list in lists:
        result += if_not(a_list, [])

    unique_results = list(set(result))

    return if_not(unique_results, None)



from setuptools import setup

#with open('requirements.txt') as f:
#    required = f.read().splitlines()

setup(name='servicemanager',
      version='0.0.16',
      description='A python tool to manage developing and testing with lots of microservices',
      url='https://github.com/hmrc/service-manager',
      author='vaughansharman',
      license='Apache Licence 2.0',
      packages=['servicemanager', 'servicemanager.actions', 'servicemanager.server', 'servicemanager.service', 'servicemanager.thirdparty'],
      install_requires=['requests==2.2.1','pymongo==2.6.3','bottle==0.12.4','pytest==2.5.2','argcomplete==0.8.1'],
      scripts=['bin/sm', 'bin/smserver'],
      zip_safe=False)



#!/usr/bin/env python

import os
import json
import time
import sys
import collections
import copy

from pymongo import Connection

from smcredentials import CredentialsResolver
from smprocess import kill_by_test_id, test_has_running_processes
from servicemanager.service.smplayservice import SmPlayService, SmPlayServiceStarter
from service.smdropwizardservice import SmDropwizardService, SmDropwizardServiceStarter
from service.smexternalservice import SmExternalService, SmExternalServiceStarter
from service.smpythonservice import SmPythonServiceStarter, SmPythonService
from smutil import pretty_print_list, if_not, unify_lists
from actions.colours import BColors


b = BColors()


def validate_always_run_from(value):
    if value in ["RELEASE", "SNAPSHOT", "SOURCE"]:
        return True
    return False


def merge_dict(d1, d2):
    """
    Modifies d1 in-place to contain values from d2.  If any value
    in d1 is a dictionary (or dict-like), *and* the corresponding
    value in d2 is also a dictionary, then merge them in-place.
    """
    for k,v2 in d2.items():
        v1 = d1.get(k) # returns None if v1 has no value for this key
        if ( isinstance(v1, collections.Mapping) and
             isinstance(v2, collections.Mapping) ):
            merge_dict(v1, v2)
        else:
            d1[k] = v2


class ServiceManagerException(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)


class SmApplication():
    # Application context object - loads, validates and stores global configuration

    def __init__(self, configuration_dir_parameter=None, features=None):

        self.workspace = SmApplication._required_environment_directory("WORKSPACE", "your workspace root dir")
        self.java_home = SmApplication._required_environment_directory("JAVA_HOME", "path/to/jdk")

        self.configuration_dir = SmApplication._get_configuration_dir(self, configuration_dir_parameter)

        self.services = SmApplication._read_json_config(self, "services.json")
        self.profiles = SmApplication._read_json_config(self, "profiles.json")
        self.config = SmApplication._read_json_config(self, "config.json")

        self.templates = SmApplication._read_json_config(self, "templates.json")
        self.merge_template_config()

        self.nexus_repo_host = SmApplication._read_json_config(self, "config.json")["nexus"]["host"]
        self.play_extraction_dir = SmApplication._read_json_config(self, "config.json")["playExtractionDir"]
        self.service_mappings = SmApplication._read_json_config(self, "service_mappings.json")
        self.features = features

        for profile in self.profiles:
            if profile in self.services:
                print b.warning + "WARNING: The profile %s conflicts with a service of the same name, this hides the profile and makes it unusable!!!" % profile + b.endc

    def service_data(self, service_name):
        if not service_name in self.services:
            raise ServiceManagerException("Unknown service: %s" % service_name)
        return self.services[service_name]

    def services_for_profile(self, profile):
        if self.is_profile(profile):
            return self.profiles[profile]
        raise ServiceManagerException("The requested profile '%s' does not exist" % profile)

    def has_service(self, service_name):
        return service_name in self.services

    def is_profile(self, profile):
        return profile in self.profiles

    def get_merged_services_config(self):
        # the services have been merged with templates at this point
        return self.services

    def merge_template_config(self):
        for service in self.services:
            template_key = "template"
            if template_key in self.services[service]:
                template_to_use = self.services[service][template_key]
                if template_to_use in self.templates:
                    copied = copy.deepcopy(self.templates[template_to_use]) #merge_dict(self.templates[template_to_use], self.services[service])
                    original_config = copy.deepcopy(self.services[service])
                    merge_dict(copied, original_config)
                    str_version = json.dumps(copied)
                    str_replaced = str_version.replace("${SERVICE-ID}", service)
                    new_config = json.loads(str_replaced)
                    self.services[service] = new_config #hehe, take that recursion! (O)1

    def describe(self):
        print "Services:"

        for service_name in collections.OrderedDict(sorted(self.services.items())):
            print "\t" + service_name.ljust(12) + " =>\t" + self.services[service_name]["name"]

        print "Profiles:"

        for profile_name in collections.OrderedDict(sorted(self.profiles.items())):
            print "\t" + profile_name.ljust(12) + " =>\t" + ", ".join(self.profiles[profile_name])

    def _read_json_config(self, filename):
        config_file = os.path.join(self.configuration_dir, filename)

        if not os.path.isfile(config_file):
            SmApplication.exit_with_error("Cannot find configuration file: %s" % config_file)

        try:
            with open(config_file) as f:
                return json.load(f)
        except Exception as e:
            SmApplication.exit_with_error("Unable to load configuration file '%s' due to exception: %s" % (config_file, str(e)))

    def _get_configuration_dir(self, configuration_dir_parameter):

        if configuration_dir_parameter:
            configuration_dir = configuration_dir_parameter
        else:
            configuration_dir = os.path.join(self.workspace, "service-manager-config")

        configuration_dir = os.path.abspath(configuration_dir)

        if not configuration_dir.endswith(os.sep):
            configuration_dir += os.sep

        if not os.path.isdir(configuration_dir):
            SmApplication.exit_with_error("Cannot find configuration directory: %s" % configuration_dir)

        return configuration_dir

    @staticmethod
    def _required_environment_directory(environment_variable, description_for_error):

        directory = os.environ.get(environment_variable, None)

        if not directory:
            SmApplication.exit_with_error("'%s' environment variable is required. You can add this to your ~/.bash_profile by adding the line %s=[%s]" %
                                          (environment_variable, environment_variable, description_for_error))

        directory = os.path.abspath(directory)

        if not os.path.isdir(directory):
            SmApplication.exit_with_error("'%s' environment variable points to non-existent directory: %s" % (environment_variable, directory))

        return directory

    @staticmethod
    def exit_with_error(message):
        print "Error: " + message
        sys.exit(1)


class SmContext():

    # Invocation context object - representing a call to sm.py, or a request received by SmServer

    MAX_TIME_TO_DIE_SECONDS = 3

    def __init__(self, application, test_id, offline=False, show_progress=True, request_specific_features=None):

        self.application = application
        self.test_id = test_id
        self.instance_id = if_not(self.test_id, "LOCAL")
        self.database_name_prefix = test_id
        self.offline = offline
        self.show_progress = show_progress
        self.is_test = test_id is not None
        self.features = unify_lists(request_specific_features, self.application.features)
        self.credentials = CredentialsResolver(self)

    def config_value(self, key, default=None):
        if key in self.application.config:
            return self.application.config[key]
        else:
            return default

    def exception(self, message):
        self.log(message)
        return ServiceManagerException(message)

    def log(self, message):
        if self.test_id:
            print "[%s] %s" % (self.test_id, message)
        else:
            print message

    def get_service(self, service_name):
        service_type = self.service_type(service_name)

        if service_type == "external":
            return SmExternalService(self, service_name)
        elif service_type == "dropwizard":
            return SmDropwizardService(self, service_name)
        elif service_type == "play":
            return SmPlayService(self, service_name)
        elif service_type == "assets":
            return SmPythonService(self, service_name)
        else:
            raise self.exception("Unknown service type '%s' for service '%s' - please check services.json" % (service_type, service_name))

    def drop_database_for_service(self, service_name):
        self._drop_database(self.database_name_prefix + "-" + service_name)

    def drop_database_for_test(self):
        self._drop_database(self.database_name_prefix)

    def _drop_database(self, database_name):
        c = Connection()
        try:
            if database_name in c.database_names():
                self.log("Dropping database: %s" % database_name)
                c.drop_database(database_name)
        finally:
            c.close()

    def service_data(self, service_name):
        return self.application.service_data(service_name)

    def get_microservice_target_path(self, service_name):
        data = self.service_data(service_name)
        return self.application.workspace + data["location"] + "/target/"

    def _create_extension(self, service_name, run_from):
        if self.service_type(service_name) == "play" or self.service_type(service_name) == "assets":
            return ".zip"
        else:
            return "-%s-shaded.jar" % run_from

    def get_jar_filename(self, service_name, run_from):
        data = self.service_data(service_name)
        binary = data["binary"]
        artifact = binary["artifact"]
        extension = self._create_extension(service_name, run_from)
        return artifact + extension

    def kill(self, service_name=None):
        if service_name:
            self._kill_service(service_name)
        elif self.test_id:
            self._kill_test()
        else:
            self.kill_everything()

    def _kill_service(self, service_name):
        service = self.get_service(service_name)
        print "killing service name: " + service_name
        service.stop()

    def _kill_test(self):
        services_stopped = kill_by_test_id(self, False)
        seconds_remaining = SmContext.MAX_TIME_TO_DIE_SECONDS
        while seconds_remaining > 0 and test_has_running_processes(self):
            time.sleep(1)
            seconds_remaining -= 1
        services_stopped |= kill_by_test_id(self, True)
        self._clean_up_services(services_stopped)

    def kill_everything(self):
        for service_name in self.application.services:
            self._kill_service(service_name)

    def _clean_up_services(self, services):
        for service_name in services:
            self.get_service(service_name).clean_up()

    def has_service(self, service_name):
        return self.application.has_service(service_name)

    def service_type(self, service_name):
        service_type = self.service_data(service_name)["type"]
        if service_type not in ["dropwizard", "play", "external", "assets"]:
            raise self.exception("Unknown service type '%s' in services.json for service '%s'" % (service_type, service_name))
        return service_type

    def get_ports_used(self):
        ports_used = {}
        for service_name in self.application.services:
            service = self.application.services[service_name]
            if "defaultPort" in service:
                ports_used[service["defaultPort"]] = service_name
                if "defaultAdminPort" in service:
                    ports_used[service["defaultAdminPort"]] = service_name
        return ports_used

    def is_profile(self, profile):
        return self.application.is_profile(profile)

    def get_run_from_service_override_value_or_use_default(self, service, original_runfrom):
        if "always_run_from" in service.service_data:
            if validate_always_run_from(service.service_data["always_run_from"]):
                self.log("Service '%s' has been overridden to always use '%s' version" % (service.service_name, service.service_data["always_run_from"]))
                return service.service_data["always_run_from"]
        return original_runfrom

    def get_service_starter(self, service_name, run_from, proxy, classifier=None, service_mapping_ports=None, port=None, admin_port=None, version=None):
        service = self.get_service(service_name)
        run_from = self.get_run_from_service_override_value_or_use_default(service, run_from)

        if not service_mapping_ports:
            service_mapping_ports = {}

        service_type = self.service_type(service_name)

        if (not self.is_test) and service.is_started_on_default_port():
            print service_name + " is already running on its default port, not starting a new one"
            return None

        if service_type == "external":
            starter = SmExternalServiceStarter(self, service_name)
        elif service_type == "dropwizard":
            starter = SmDropwizardServiceStarter(self, service_name, run_from, port, admin_port, classifier, service_mapping_ports, version, proxy)
        elif service_type == "play":
            starter = SmPlayServiceStarter(self, service_name, run_from, port, classifier, service_mapping_ports, version, proxy)
        elif service_type == "assets":
            starter = SmPythonServiceStarter(self, service_name, run_from, port, classifier, service_mapping_ports, version, None)
        else:
            raise self.exception("Unknown service type '%s' for service '%s' - please check services.json" % (service_type, service_name))

        return starter

    def start_service(self, service_name, run_from, proxy, classifier=None, service_mapping_ports=None, port=None, admin_port=None, version=None):
        service_starter = self.get_service_starter(service_name, run_from, proxy, classifier, service_mapping_ports, port, admin_port, version)
        feature_string = pretty_print_list(" with feature$s $list enabled", self.features)
        self.log("Starting '%s' from %s%s..." % (service_name, run_from, feature_string))
        service_process_id = service_starter.start()

        if service_process_id:
            feature_string = pretty_print_list(" and feature$s $list enabled", self.features)
            self.log("'%s' version '%s' started with PID = %d%s" % (service_name, version, service_process_id, feature_string))
        else:
            self.log("'%s' version does not appear to have started" % service_name)

        return service_process_id



#!/usr/bin/env python
from signal import SIGINT, SIGKILL
import os
import re

from servicemanager import subprocess


def kill_pid(context, pid, force=False):

    if _is_system_or_smserver_or_test_process(pid):
        return "Not allowed to kill system, test or smserver process (pid = %d)" % pid

    try:
        if force:
            os.kill(pid, SIGKILL)
        else:
            os.kill(pid, SIGINT)

    except Exception, e:
        error = "Could not kill pid: " + str(pid) + " because of exception: " + str(e)
        context.log(error)
        return error


def _is_system_or_smserver_or_test_process(pid):

    if pid == 1:
        return True

    if _is_init_process(pid):
        return True

    if _is_smserver_process(pid):
        return True

    if _is_pytest_process(pid):
        return True

    if _is_pycharm_process(pid):
        return True

    if _is_pycharm_test_process(pid):
        return True

    if _is_pycharm_related_process(pid):
        return True

    return False


def _is_init_process(pid):
    command = "ps -eo ppid,pid,etime,rss,args | grep 'init --user' | grep -v 'grep init --user' | awk '{print $2 }'"
    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    ps_output = ps_command.stdout.read()
    ps_pid_str = ps_output.strip()
    if ps_pid_str and int(ps_pid_str) == pid:
        return True
    return False


def _is_smserver_process(pid):
    command = "ps -eo pid,args | grep %d | grep 'smserver\.py' | awk '{print $1}'" % pid
    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    ps_output = ps_command.stdout.read()
    ps_pid_str = ps_output.strip()
    if ps_pid_str and int(ps_pid_str) == pid:
        return True
    return False


def _is_pytest_process(pid):
    command = "ps -eo pid,args | grep %d | grep 'py\.test' | awk '{print $1}'" % pid
    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    ps_output = ps_command.stdout.read()
    ps_pid_str = ps_output.strip()
    if ps_pid_str and int(ps_pid_str) == pid:
        return True
    return False


def _is_pycharm_test_process(pid):
    command = "ps -eo pid,args | grep %d | grep 'pytestrunner\.py' | awk '{print $1}'" % pid
    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    ps_output = ps_command.stdout.read()
    ps_pid_str = ps_output.strip()
    if ps_pid_str and int(ps_pid_str) == pid:
        return True
    return False


def _is_pycharm_process(pid):
    command = "ps -eo pid,args | grep %d | grep 'pydevd\.py' | awk '{print $1}'" % pid
    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    ps_output = ps_command.stdout.read()
    ps_pid_str = ps_output.strip()
    if ps_pid_str and int(ps_pid_str) == pid:
        return True
    return False

def _is_pycharm_related_process(pid):
    command = "ps -eo pid,args | grep %d | grep 'pycharm' | awk '{print $1}'" % pid
    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    ps_output = ps_command.stdout.read()
    ps_pid_str = ps_output.strip()
    if ps_pid_str and int(ps_pid_str) == pid:
        return True
    return False


def kill_by_test_id(context, force):
    pids = _get_process_ids_for_test(context)
    services_killed = set()
    for pid in pids:
        service_name = _get_service_name_for_pid(pid)
        if force:
            if service_name:
                context.log("Force killing %s (pid: %s)" % (service_name, pid))    
            else:
                context.log("Force killing pid: %s (unknown/missing service name)" % pid)
        context.log("killing %s" % service_name)
        kill_pid(context, pid, force)
        if service_name:
            services_killed.add(service_name)

    return services_killed


def test_has_running_processes(context):
    return len(_get_process_ids_for_test(context)) > 0


def _get_process_ids_for_test(context):
    return _get_process_ids_for_processes_matching(re.escape("-Dservice.manager.testId=%s" % context.instance_id))


def _get_process_ids_for_processes_matching(regex):
    command = "ps -eo pid,args | grep '%s' |grep -v 'grep %s' |  awk '{print $1}'" % (regex, regex)
    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    stdout, stderr = ps_command.communicate()

    return map(int, stdout.split("\n")[:-1])


def _get_service_name_for_pid(pid):
    command = "ps -p %s -o command,args" % pid

    ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    stdout, stderr = ps_command.communicate()
    ps_output = stdout.split()

    p = re.compile('-Dservice\.manager\.serviceName=([A-Z_]+)')

    for i in ps_output:
        t = p.match(i)
        if t:
            return t.group(1)

    return None


class SmProcess:

    def __init__(self, ppid, pid, uptime, mem, args):
        self.ppid = ppid
        self.pid = pid
        self.uptime = uptime
        self.mem = mem
        self.args = args

    @staticmethod
    def processes_matching(regex):
        command = "ps -eo ppid,pid,etime,rss,args | egrep -e '%s' | grep -vi 'grep -e'" % regex
        ps_command = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
        stdout, stderr = ps_command.communicate()
        ps_output = stdout.split("\n")[:-1]

        def process_line_to_object(process_line):
            values = process_line.strip().split()
            return SmProcess(int(values[0]), int(values[1]), values[2], values[3], values[4:])

        ret = map(process_line_to_object, ps_output)

        return ret

    def has_argument(self, argument):
        return argument in self.args

    def extract_argument(self, regex_with_group, default_if_none):
        regex = re.compile(regex_with_group)
        for arg in self.args:
            match = regex.match(arg)
            if match:
                return match.group(1)

        return default_if_none

    def extract_arguments(self, regex_with_group, default_if_none):
        regex = re.compile(regex_with_group)
        matches = []
        for arg in self.args:
            match = regex.match(arg)
            if match:
                matches += [match.group(1)]

        if matches:
            return ",".join(matches)

        return default_if_none

    def extract_integer_argument(self, regex_with_group, default_if_none):
        arg = self.extract_argument(regex_with_group, None)

        if arg:
            return int(arg)

        return default_if_none



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import socket

from servicemanager.thirdparty.atomicinteger import AtomicInteger


class PortProvider:

    def __init__(self):
        self._port_counter = AtomicInteger(10000, 10000, 65535)

    @staticmethod
    def _is_available(port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect(("127.0.0.1", int(port)))
            s.shutdown(2)
            return False
        except Exception:
            return True

    def next_available_port(self):
        next_port_value = self._port_counter.increment_and_get()

        while not self._is_available(next_port_value):
            next_port_value = self._port_counter.increment_and_get()

        return next_port_value
